@@ -5,10 +5,12 @@ from threading import Thread, RLock
 from thread import get_ident
 import sys
 from time import sleep
+import select
+import socket
 
 import zmq
+from zmq.eventloop import ioloop
 
-from circus._zmq import ioloop
 from circus.controller import Controller
 from circus.exc import AlreadyExist
 from circus import logger
@@ -18,7 +20,9 @@ from circus.config import get_config
 from circus.plugins import get_plugin_cmd
 from circus.sockets import CircusSocket, CircusSockets
 
-import select
+
+class ReloadArbiterException(Exception):
+    pass
 
 
 class Arbiter(object):
@@ -61,6 +65,8 @@ class Arbiter(object):
     - **debug** -- if True, adds a lot of debug info in the stdout (default:
       False)
     - **proc_name** -- the arbiter process name
+    - **fqdn_prefix** -- a prefix for the unique identifier of the circus
+                         instance on the cluster.
     """
     def __init__(self, watchers, endpoint, pubsub_endpoint, check_delay=.5,
                  prereload_fn=None, context=None, loop=None, statsd=False,
@@ -70,7 +76,7 @@ class Arbiter(object):
                  httpd_host='localhost', httpd_port=8080,
                  httpd_close_outputs=False, debug=False,
                  ssh_server=None, proc_name='circusd', pidfile=None,
-                 loglevel=None, logoutput=None):
+                 loglevel=None, logoutput=None, fqdn_prefix=None):
         self.watchers = watchers
         self.endpoint = endpoint
         self.check_delay = check_delay
@@ -78,25 +84,33 @@ class Arbiter(object):
         self.pubsub_endpoint = pubsub_endpoint
         self.multicast_endpoint = multicast_endpoint
         self.proc_name = proc_name
+        self.ssh_server = ssh_server
 
         self.pidfile = pidfile
         self.loglevel = loglevel
         self.logoutput = logoutput
 
+        socket_fqdn = socket.getfqdn()
+        if fqdn_prefix is None:
+            fqdn = socket_fqdn
+        else:
+            fqdn = '{}@{}'.format(fqdn_prefix, socket_fqdn)
+        self.fqdn = fqdn
+
         self.ctrl = self.loop = None
         self.socket_event = False
 
         # initialize zmq context
-        self.context = context or zmq.Context.instance()
+        self._init_context(context)
         self.pid = os.getpid()
         self._watchers_names = {}
         self.alive = True
         self._lock = RLock()
         self.debug = debug
         if self.debug:
-            stdout_stream = stderr_stream = {'class': 'StdoutStream'}
+            self.stdout_stream = self.stderr_stream = {'class': 'StdoutStream'}
         else:
-            stdout_stream = stderr_stream = None
+            self.stdout_stream = self.stderr_stream = None
 
         # initializing circusd-stats as a watcher when configured
         self.statsd = statsd
@@ -114,8 +128,8 @@ class Arbiter(object):
                 cmd += ' --log-level DEBUG'
             stats_watcher = Watcher('circusd-stats', cmd, use_sockets=True,
                                     singleton=True,
-                                    stdout_stream=stdout_stream,
-                                    stderr_stream=stderr_stream,
+                                    stdout_stream=self.stdout_stream,
+                                    stderr_stream=self.stderr_stream,
                                     copy_env=True, copy_path=True,
                                     close_child_stderr=statsd_close_outputs,
                                     close_child_stdout=statsd_close_outputs)
@@ -133,8 +147,8 @@ class Arbiter(object):
 
             httpd_watcher = Watcher('circushttpd', cmd, use_sockets=True,
                                     singleton=True,
-                                    stdout_stream=stdout_stream,
-                                    stderr_stream=stderr_stream,
+                                    stdout_stream=self.stdout_stream,
+                                    stderr_stream=self.stderr_stream,
                                     copy_env=True, copy_path=True,
                                     close_child_stderr=httpd_close_outputs,
                                     close_child_stdout=httpd_close_outputs)
@@ -150,35 +164,193 @@ class Arbiter(object):
                 sockets.append(httpd_socket)
 
         # adding each plugin as a watcher
-        ch_stderr = stderr_stream is None
-        ch_stdout = stdout_stream is None
+        ch_stderr = self.stderr_stream is None
+        ch_stdout = self.stdout_stream is None
 
         if plugins is not None:
             for plugin in plugins:
-                fqnd = plugin['use']
-                name = 'plugin:%s' % fqnd.replace('.', '-')
+                fqn = plugin['use']
                 cmd = get_plugin_cmd(plugin, self.endpoint,
                                      self.pubsub_endpoint, self.check_delay,
                                      ssh_server, debug=self.debug)
-                plugin_watcher = Watcher(name, cmd, priority=1, singleton=True,
-                                         stdout_stream=stdout_stream,
-                                         stderr_stream=stderr_stream,
-                                         copy_env=True, copy_path=True,
-                                         close_child_stderr=ch_stderr,
-                                         close_child_stdout=ch_stdout)
+                plugin_cfg = dict(cmd=cmd, priority=1, singleton=True,
+                                  stdout_stream=self.stdout_stream,
+                                  stderr_stream=self.stderr_stream,
+                                  copy_env=True, copy_path=True,
+                                  close_child_stderr=ch_stderr,
+                                  close_child_stdout=ch_stdout)
+                plugin_cfg.update(plugin)
+                if 'name' not in plugin_cfg:
+                    plugin_cfg['name'] = fqn
 
+                plugin_watcher = Watcher.load_from_config(plugin_cfg)
                 self.watchers.append(plugin_watcher)
 
         self.sockets = CircusSockets(sockets)
         self.warmup_delay = warmup_delay
+
+    def _init_context(self, context):
+        self.context = context or zmq.Context.instance()
         self.loop = ioloop.IOLoop.instance()
         self.ctrl = Controller(self.endpoint, self.multicast_endpoint,
                                self.context, self.loop, self, self.check_delay)
 
+    def get_socket(self, name):
+        return self.sockets.get(name, None)
+
+    def get_socket_config(self, config, name):
+        for i in config.get('sockets', []):
+            if i['name'] == name:
+                return i.copy()
+        return None
+
+    def get_watcher_config(self, config, name):
+        for i in config.get('watchers', []):
+            if i['name'] == name:
+                return i.copy()
+        return None
+
+    def get_plugin_config(self, config, name):
+        for i in config.get('plugins', []):
+            if i['name'] == name:
+                cfg = i.copy()
+                cmd = get_plugin_cmd(cfg, self.endpoint,
+                                     self.pubsub_endpoint, self.check_delay,
+                                     self.ssh_server, debug=self.debug)
+
+                cfg.update(dict(cmd=cmd, priority=1, singleton=True,
+                                stdout_stream=self.stdout_stream,
+                                stderr_stream=self.stderr_stream,
+                                copy_env=True, copy_path=True))
+                return cfg
+        return None
+
+    @classmethod
+    def get_arbiter_config(cls, config):
+        cfg = config.copy()
+        del cfg['watchers']
+        del cfg['plugins']
+        del cfg['sockets']
+
+        return cfg
+
+    def reload_from_config(self, config_file=None):
+        new_cfg = get_config(config_file if config_file else self.config_file)
+        # if arbiter is changed, reload everything
+        if self.get_arbiter_config(new_cfg) != self._cfg:
+            raise ReloadArbiterException
+
+        # Gather socket names.
+        current_sn = set([i.name for i in self.sockets.values()])
+        new_sn = set([i['name'] for i in new_cfg.get('sockets', [])])
+        added_sn = new_sn - current_sn
+        deleted_sn = current_sn - new_sn
+        maybechanged_sn = current_sn - deleted_sn
+        changed_sn = set([])
+        wn_with_changed_socket = set([])
+        wn_with_deleted_socket = set([])
+
+        # get changed sockets
+        for n in maybechanged_sn:
+            s = self.get_socket(n)
+            if self.get_socket_config(new_cfg, n) != s._cfg:
+                changed_sn.add(n)
+
+                # just delete the socket and add it again
+                deleted_sn.add(n)
+                added_sn.add(n)
+
+                # Get the watchers whichs use these, so they could be
+                # deleted and added also
+                for w in self.iter_watchers():
+                    if 'circus.sockets.%s' % n.lower() in w.cmd:
+                        wn_with_changed_socket.add(w.name)
+
+        # get deleted sockets
+        for n in deleted_sn:
+            s = self.get_socket(n)
+            s.close()
+            # Get the watchers whichs use these, these should not be
+            # active anymore
+            for w in self.iter_watchers():
+                if 'circus.sockets.%s' % n.lower() in w.cmd:
+                    wn_with_deleted_socket.add(w.name)
+            del self.sockets[s.name]
+
+        # get added sockets
+        for n in added_sn:
+            socket_config = self.get_socket_config(new_cfg, n)
+            s = CircusSocket.load_from_config(socket_config)
+            s.bind_and_listen()
+            self.sockets[s.name] = s
+
+        if added_sn or deleted_sn:
+            # make sure all existing watchers get the new sockets in
+            # their attributes and get the old removed
+            # XXX: is this necessary? self.sockets is an mutable
+            # object
+            for watcher in self.iter_watchers():
+                # XXX: What happens as initalize is called on a
+                # running watcher?
+                watcher.initialize(self.evpub_socket, self.sockets, self)
+
+        # Gather watcher names.
+        current_wn = set([i.name for i in self.iter_watchers()])
+        new_wn = set([i['name'] for i in new_cfg.get('watchers', [])])
+        new_wn = new_wn | set([i['name'] for i in new_cfg.get('plugins', [])])
+        added_wn = (new_wn - current_wn) | wn_with_changed_socket
+        deleted_wn = current_wn - new_wn - wn_with_changed_socket
+        maybechanged_wn = current_wn - deleted_wn
+        changed_wn = set([])
+
+        if wn_with_deleted_socket and wn_with_deleted_socket not in new_wn:
+            raise ValueError('Watchers %s uses a socket which is deleted' %
+                             wn_with_deleted_socket)
+
+        # get changed watchers
+        for n in maybechanged_wn:
+            w = self.get_watcher(n)
+            new_watcher_cfg = (self.get_watcher_config(new_cfg, n) or
+                               self.get_plugin_config(new_cfg, n))
+            old_watcher_cfg = w._cfg.copy()
+            if new_watcher_cfg != old_watcher_cfg:
+                num_procs = new_watcher_cfg.get('numprocesses')
+                old_watcher_cfg['numprocesses'] = num_procs
+
+                if new_watcher_cfg == old_watcher_cfg:
+                    # if nothing but the number of processes is
+                    # changed, just changes this
+                    w.set_numprocesses(int(num_procs))
+                    continue
+
+                # Others things are changed. Just delete and add the watcher.
+                changed_wn.add(n)
+                deleted_wn.add(n)
+                added_wn.add(n)
+
+        # delete watchers
+        for n in deleted_wn:
+            w = self.get_watcher(n)
+            w.stop()
+            del self._watchers_names[w.name.lower()]
+            self.watchers.remove(w)
+
+        # add watchers
+        for n in added_wn:
+            new_watcher_cfg = (self.get_plugin_config(new_cfg, n) or
+                               self.get_watcher_config(new_cfg, n))
+
+            w = Watcher.load_from_config(new_watcher_cfg)
+            w.initialize(self.evpub_socket, self.sockets, self)
+            self.start_watcher(w)
+            self.watchers.append(w)
+            self._watchers_names[w.name.lower()] = w
+
+        return False
+
     @classmethod
     def load_from_config(cls, config_file):
         cfg = get_config(config_file)
-
         watchers = []
         for watcher in cfg.get('watchers', []):
             watchers.append(Watcher.load_from_config(watcher))
@@ -213,7 +385,13 @@ class Arbiter(object):
                       ssh_server=cfg.get('ssh_server', None),
                       pidfile=cfg.get('pidfile', None),
                       loglevel=cfg.get('loglevel', None),
-                      logoutput=cfg.get('logoutput', None))
+                      logoutput=cfg.get('logoutput', None),
+                      fqdn_prefix=cfg.get('fqdn_prefix', None))
+
+        # store the cfg which will be used, so it can be used later
+        # for checking if the cfg has been changed
+        arbiter._cfg = cls.get_arbiter_config(cfg)
+        arbiter.config_file = config_file
 
         return arbiter
 
@@ -284,19 +462,15 @@ class Arbiter(object):
         finally:
             self.ctrl.stop()
             self.evpub_socket.close()
+            if len(self.sockets) > 0:
+                self.sockets.close_all()
 
     def stop(self):
-        self._stop()
-
-    def _stop(self):
-        if self.alive:
-            self.stop_watchers(stop_alive=True)
-
+        self.stop_watchers(stop_alive=True)
+        # this will stop the loop and the closing
+        # will finish in .start()
         self.loop.stop()
 
-        # close sockets
-        self.sockets.close_all()
-
     def reap_processes(self):
         # map watcher to pids
         watchers_pids = {}
@@ -457,6 +631,6 @@ class ThreadedArbiter(Arbiter, Thread):
         return Arbiter.start(self)
 
     def stop(self):
-        self.loop.add_callback(self._stop)
-        if get_ident() != self.ident:
+        Arbiter.stop(self)
+        if get_ident() != self.ident and self.isAlive():
             self.join()
