@@ -1,16 +1,13 @@
 import glob
 import os
 import warnings
+from fnmatch import fnmatch
+from collections import defaultdict
 
 from circus import logger
 from circus.util import (DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB,
                          DEFAULT_ENDPOINT_MULTICAST, DEFAULT_ENDPOINT_STATS,
-                         StrictConfigParser, parse_env_str)
-try:
-    import gevent       # NOQA
-    DEFAULT_STREAM = 'gevent'
-except ImportError:
-    DEFAULT_STREAM = 'thread'
+                         StrictConfigParser, parse_env_str, replace_gnu_args)
 
 
 def watcher_defaults():
@@ -53,18 +50,46 @@ def to_boolean(value):
 
 
 class DefaultConfigParser(StrictConfigParser):
+
+    def __init__(self, *args, **kw):
+        StrictConfigParser.__init__(self, *args, **kw)
+        self._env = dict(os.environ)
+
+    def set_env(self, env):
+        self._env = dict(env)
+
+    def toboolean(self, value):
+        if value.lower() not in self._boolean_states:
+            raise ValueError('Not a boolean: %s' % value)
+        return self._boolean_states[value.lower()]
+
+    def get(self, section, option):
+        res = StrictConfigParser.get(self, section, option)
+        return replace_gnu_args(res, env=self._env)
+
+    def items(self, section, noreplace=False):
+        items = StrictConfigParser.items(self, section)
+        if noreplace:
+            return items
+        env = dict(os.environ)
+        return [(key, replace_gnu_args(value, env=env))
+                for key, value in items]
+
     def dget(self, section, option, default=None, type=str):
         if not self.has_option(section, option):
             return default
-        if type is str:
-            return self.get(section, option)
-        elif type is int:
-            return self.getint(section, option)
+
+        value = self.get(section, option)
+
+        if type is int:
+            value = int(value)
         elif type is bool:
-            return self.getboolean(section, option)
-        else:
+            value = self.toboolean(value)
+        elif type is not str:
             raise NotImplementedError()
 
+        return value
+
 
 def read_config(config_path):
     cfg = DefaultConfigParser()
@@ -82,9 +107,9 @@ def read_config(config_path):
 
         paths = glob.glob(filename)
         if paths == []:
-            raise IOError('%r does not lead to any config. Make sure '
-                          'include paths are relative to the main config '
-                          'file')
+            logger.warn('%r does not lead to any config. Make sure '
+                        'include paths are relative to the main config '
+                        'file' % filename)
         includes += paths
 
     for include_file in cfg.dget('circus', 'include', '').split():
@@ -106,6 +131,17 @@ def get_config(config_file):
     dget = cfg.dget
     config = {}
 
+    # reading the global environ first
+    def _upper(items):
+        return [(key.upper(), value) for key, value in items]
+
+    global_env = dict(_upper(os.environ.items()))
+
+    if 'env' in cfg.sections():
+        global_env.update(dict(_upper(cfg.items('env'))))
+
+    cfg.set_env(global_env)
+
     # main circus options
     config['check'] = dget('circus', 'check_delay', 5, int)
     config['endpoint'] = dget('circus', 'endpoint', DEFAULT_ENDPOINT_DEALER)
@@ -132,10 +168,10 @@ def get_config(config_file):
     config['pidfile'] = dget('circus', 'pidfile')
     config['loglevel'] = dget('circus', 'loglevel')
     config['logoutput'] = dget('circus', 'logoutput')
+    config['fqdn_prefix'] = dget('circus', 'fqdn_prefix', None, str)
 
     # Initialize watchers, plugins & sockets to manage
     watchers = []
-    environs = {}
     plugins = []
     sockets = []
 
@@ -146,14 +182,16 @@ def get_config(config_file):
             sockets.append(sock)
 
         if section.startswith("plugin:"):
-            plugins.append(dict(cfg.items(section)))
+            plugin = dict(cfg.items(section))
+            plugin['name'] = section
+            plugins.append(plugin)
 
         if section.startswith("watcher:"):
             watcher = watcher_defaults()
             watcher['name'] = section.split("watcher:", 1)[1]
 
             # create watcher options
-            for opt, val in cfg.items(section):
+            for opt, val in cfg.items(section, noreplace=True):
                 if opt == 'cmd':
                     watcher['cmd'] = val
                 elif opt == 'args':
@@ -242,19 +280,45 @@ def get_config(config_file):
 
             watchers.append(watcher)
 
+    # Second pass to make sure env sections apply to all watchers.
+    environs = defaultdict(dict)
+
+    # global env first
+    def _extend(target, source):
+        for name, value in source:
+            if name in target:
+                continue
+            target[name] = value
+
+    for watcher in watchers:
+        _extend(environs[watcher['name']], global_env.items())
+
+    # then per-watcher env
+    for section in cfg.sections():
         if section.startswith('env:'):
-            for watcher in section.split("env:", 1)[1].split(','):
-                watcher = watcher.strip()
-                if not watcher in environs:
-                    environs[watcher] = dict()
-                environs[watcher].update([(k.upper(), v)
-                                          for k, v in cfg.items(section)])
+            section_elements = section.split("env:", 1)[1]
+            watcher_patterns = [s.strip() for s in section_elements.split(',')]
+
+            for pattern in watcher_patterns:
+                match = [w for w in watchers if fnmatch(w['name'], pattern)]
+
+                for watcher in match:
+                    watcher_name = watcher['name']
+                    extra = cfg.items(section, noreplace=True)
+                    environs[watcher_name].update(_upper(extra))
 
     for watcher in watchers:
         if watcher['name'] in environs:
             if not 'env' in watcher:
                 watcher['env'] = dict()
-            watcher['env'].update(environs[watcher['name']])
+            _extend(watcher['env'], environs[watcher['name']].items())
+
+        for option, value in watcher.items():
+            if option in ('name', 'env'):
+                continue
+            if not isinstance(value, str):
+                continue
+            watcher[option] = replace_gnu_args(value, env=watcher['env'])
 
     config['watchers'] = watchers
     config['plugins'] = plugins
