@@ -9,9 +9,9 @@ from urlparse import urlparse
 
 import zmq
 from zmq.utils.jsonapi import jsonmod as json
+from zmq.eventloop import ioloop, zmqstream
 
 from circus.util import create_udp_socket
-from circus._zmq import ioloop, zmqstream
 from circus.commands import get_commands, ok, error, errors
 from circus import logger
 from circus.exc import MessageError
@@ -29,15 +29,21 @@ class Controller(object):
         self.context = context
         self.loop = loop
         self.check_delay = check_delay * 1000
-
-        self.jobs = Queue()
+        self.started = False
 
         # initialize the sys handler
-        self.sys_hdl = SysHandler(self)
+        self._init_syshandler()
 
         # get registered commands
         self.commands = get_commands()
 
+    def _init_syshandler(self):
+        self.sys_hdl = SysHandler(self)
+
+    def _init_stream(self):
+        self.stream = zmqstream.ZMQStream(self.ctrl_socket, self.loop)
+        self.stream.on_recv(self.handle_message)
+
     def initialize(self):
         # initialize controller
 
@@ -45,8 +51,7 @@ class Controller(object):
         self.ctrl_socket = self.context.socket(zmq.ROUTER)
         self.ctrl_socket.bind(self.endpoint)
         self.ctrl_socket.linger = 0
-        self.stream = zmqstream.ZMQStream(self.ctrl_socket, self.loop)
-        self.stream.on_recv(self.handle_message)
+        self._init_stream()
 
         # Initialize UDP Socket
         multicast_addr, multicast_port = urlparse(self.multicast_endpoint)\
@@ -58,30 +63,21 @@ class Controller(object):
 
     def start(self):
         self.initialize()
-        self.caller = ioloop.PeriodicCallback(self.wakeup, self.check_delay,
-                                              self.loop)
+        self.caller = ioloop.PeriodicCallback(self.arbiter.manage_watchers,
+                                              self.check_delay, self.loop)
         self.caller.start()
+        self.started = True
 
     def stop(self):
-        self.caller.stop()
-        self.stream.flush()
-        self.stream.close()
-        self.ctrl_socket.close()
-
-    def wakeup(self):
-        job = None
-        try:
-            job = self.jobs.get(block=False)
-        except Empty:
-            pass
-
-        if job is not None:
-            self.dispatch(job)
-        self.arbiter.manage_watchers()
-
-    def add_job(self, cid, msg):
-        self.jobs.put((cid, msg), False)
-        self.wakeup()
+        if self.started:
+            self.caller.stop()
+            try:
+                self.stream.flush()
+                self.stream.close()
+            except (IOError, zmq.ZMQError):
+                pass
+            self.ctrl_socket.close()
+        self.sys_hdl.stop()
 
     def handle_message(self, raw_msg):
         cid, msg = raw_msg
@@ -93,6 +89,9 @@ class Controller(object):
             logger.debug("got message %s", msg)
             self.add_job(cid, msg)
 
+    def add_job(self, cid, msg):
+        self.loop.add_callback(self.dispatch, (cid, msg))
+
     def handle_autodiscover_message(self, fd_no, type):
         data, address = self.udp_socket.recvfrom(1024)
         data = json.loads(data)
