@@ -13,7 +13,6 @@ from circus import get_arbiter
 from circus.util import (DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB,
                          DEFAULT_ENDPOINT_STATS)
 from circus.client import CircusClient, make_message
-from circus._zmq import ioloop
 
 
 def resolve_name(name):
@@ -27,7 +26,7 @@ def resolve_name(name):
         try:
             ret = __import__('.'.join(module_name))
             break
-        except ImportError, exc:
+        except ImportError as exc:
             last_exc = exc
             if cursor == 0:
                 raise
@@ -87,13 +86,16 @@ class TestCircus(unittest.TestCase):
         worker = {'cmd': _CMD, 'args': args, 'working_dir': wdir,
                   'name': 'test', 'graceful_timeout': 4}
         worker.update(kw)
+        debug = kw.get('debug', False)
+
         if stats:
             arbiter = get_arbiter([worker], background=True, plugins=plugins,
                                   stats_endpoint=DEFAULT_ENDPOINT_STATS,
-                                  debug=kw.get('debug', False))
+                                  statsd=True,
+                                  debug=debug, statsd_close_outputs=not debug)
         else:
             arbiter = get_arbiter([worker], background=True, plugins=plugins,
-                                  debug=kw.get('debug', False))
+                                  debug=debug)
         arbiter.start()
         return testfile, arbiter
 
@@ -143,7 +145,7 @@ class Process(object):
 
     def handle_quit(self, *args):
         self._write('QUIT')
-        sys.exit(0)
+        self.alive = False
 
     def handle_chld(self, *args):
         self._write('CHLD')
@@ -166,19 +168,23 @@ class TimeoutException(Exception):
     pass
 
 
-def poll_for(filename, needle, timeout=5):
+def poll_for(filename, needles, timeout=5):
     """Poll a file for a given string.
 
     Raises a TimeoutException if the string isn't found after timeout seconds
     of polling.
 
     """
+    if isinstance(needles, str):
+        needles = [needles]
+
     start = time()
-    while (time() - start) < 5:
+    while time() - start < 5:
         with open(filename) as f:
             content = f.read()
-        if needle in content:
-            return True
+        for needle in needles:
+            if needle in content:
+                return True
         # When using gevent this will make sure the redirector greenlets are
         # scheduled.
         sleep(0)
@@ -212,7 +218,7 @@ def run_plugin(klass, config, duration=300):
                    **config)
     plugin.statsd = _statsd
 
-    end = ioloop.DelayedCallback(plugin.loop.stop, duration, plugin.loop)
-    end.start()
+    deadline = time() + (duration / 1000.)
+    plugin.loop.add_timeout(deadline, plugin.loop.stop)
     plugin.start()
     return _statsd
