@@ -1,6 +1,8 @@
 import unittest
 import os
+from mock import patch
 
+from circus import logger
 from circus.config import get_config
 from circus.watcher import Watcher
 from circus.process import Process
@@ -19,8 +21,11 @@ _CONF = {
     'env_var': os.path.join(HERE, 'env_var.ini'),
     'env_section': os.path.join(HERE, 'env_section.ini'),
     'multiple_wildcard': os.path.join(HERE, 'multiple_wildcard.ini'),
+    'empty_include': os.path.join(HERE, 'empty_include.ini'),
     'circus': os.path.join(HERE, 'circus.ini'),
-    'nope': os.path.join(HERE, 'nope.ini')
+    'nope': os.path.join(HERE, 'nope.ini'),
+    'unexistant': os.path.join(HERE, 'unexistant.ini'),
+    'issue442': os.path.join(HERE, 'issue442.ini')
 }
 
 
@@ -70,6 +75,15 @@ class TestConfig(unittest.TestCase):
         watchers = conf['watchers']
         self.assertEquals(len(watchers), 3)
 
+    @patch.object(logger, 'warn')
+    def test_empty_include(self, mock_logger_warn):
+        """https://github.com/mozilla-services/circus/pull/473"""
+        try:
+            get_config(_CONF['empty_include'])
+        except:
+            self.fail('Non-existent includes should not raise')
+        self.assertTrue(mock_logger_warn.called)
+
     def test_watcher_graceful_timeout(self):
         conf = get_config(_CONF['issue210'])
         watcher = Watcher.load_from_config(conf['watchers'][0])
@@ -102,6 +116,15 @@ class TestConfig(unittest.TestCase):
             self.assertEquals("%s:/bin" % os.getenv('PATH'),
                               watcher.env['PATH'])
 
+        self.assertEquals('test1', watcher1.env['TEST1'])
+        self.assertEquals('test1', watcher2.env['TEST1'])
+
+        self.assertEquals('test2', watcher1.env['TEST2'])
+        self.assertEquals('test2', watcher2.env['TEST2'])
+
+        self.assertEquals('test3', watcher1.env['TEST3'])
+        self.assertEquals('test3', watcher2.env['TEST3'])
+
     def test_issue395(self):
         conf = get_config(_CONF['issue395'])
         watcher = conf['watchers'][0]
@@ -125,8 +148,23 @@ class TestConfig(unittest.TestCase):
         self.assertEquals(len(watchers), 3)
         watchers = conf['watchers']
         watchers.sort()
-        self.assertEquals(watchers[2]['env'], {'INI': 'private.ini'})
+        self.assertEquals(watchers[2]['env']['INI'], 'private.ini')
         self.assertEqual(conf['check'], 555)
 
     def test_config_unexistant(self):
-        self.assertRaises(IOError, get_config, _CONF['nope'])
+        self.assertRaises(IOError, get_config, _CONF['unexistant'])
+
+    def test_variables_everywhere(self):
+        os.environ['circus_stats_endpoint'] = 'tcp://0.0.0.0:9876'
+        os.environ['circus_statsd'] = 'True'
+
+        # these will be overriden
+        os.environ['circus_uid'] = 'ubuntu'
+        os.environ['circus_gid'] = 'ubuntu'
+
+        conf = get_config(_CONF['issue442'])
+
+        self.assertEqual(conf['stats_endpoint'], 'tcp://0.0.0.0:9876')
+        self.assertTrue(conf['statsd'])
+        self.assertEqual(conf['watchers'][0]['uid'], 'tarek')
+        self.assertEqual(conf['watchers'][0]['gid'], 'root')
