@@ -5,7 +5,7 @@ import os
 
 from circus.stats.collector import SocketStatsCollector
 from circus.tests.support import unittest
-from circus._zmq import ioloop
+from zmq.eventloop import ioloop
 
 
 TRAVIS = os.getenv('TRAVIS', False)
@@ -86,4 +86,4 @@ class TestSocketCollector(unittest.TestCase):
 
         stat = streamer.stats[0]
         self.assertTrue(stat['fd'] in fds)
-        self.assertTrue(stat['reads'] > 9)
+        self.assertTrue(stat['reads'] > 1)
