@@ -3,12 +3,15 @@ import sys
 import os
 import threading
 import time
+from test.test_support import captured_output
 
-from circus._zmq import ioloop
+from zmq.eventloop import ioloop
 from circus.tests.support import TestCircus, poll_for, truncate_file
 from circus.stream import QueueStream
 from circus.watcher import Watcher
 from circus.process import UNEXISTING
+from circus import logger
+from circus.util import get_python_version
 
 import warnings
 
@@ -25,9 +28,11 @@ class TestWatcher(TestCircus):
         dummy_process = 'circus.tests.support.run_process'
         cls.stream = QueueStream()
         testfile, arbiter = cls._create_circus(
-            dummy_process, stdout_stream={'stream': cls.stream})
+            dummy_process, stdout_stream={'stream': cls.stream},
+            debug=True)
         cls.arbiter = arbiter
         cls.test_file = testfile
+        poll_for(testfile, 'START')
 
     @classmethod
     def tearDownClass(cls):
@@ -44,6 +49,14 @@ class TestWatcher(TestCircus):
     def pids(self):
         return self.call('list', name='test').get('pids')
 
+    def test_decr_too_much(self):
+        res = self.numprocesses('decr', name='test', nb=100)
+        self.assertEqual(res, 0)
+        res = self.numprocesses('decr', name='test', nb=100)
+        self.assertEqual(res, 0)
+        res = self.numprocesses('incr', name='test', nb=1)
+        self.assertEqual(res, 1)
+
     def test_signal(self):
         self.assertEquals(self.numprocesses('incr', name='test'), 2)
         # wait for both to have started
@@ -68,11 +81,16 @@ class TestWatcher(TestCircus):
         watcher = self.arbiter.get_watcher("test")
 
         to_kill = []
-        self.assertEquals(len(watcher.processes), 2)
+        nb_proc = len(watcher.processes)
+
         for process in watcher.processes.values():
             to_kill.append(process.pid)
             # the process is killed in an unsual way
-            os.kill(process.pid, signal.SIGSEGV)
+            try:
+                os.kill(process.pid, signal.SIGSEGV)
+            except OSError:
+                pass
+
             # and wait for it to die
             try:
                 pid, status = os.waitpid(process.pid, 0)
@@ -86,7 +104,7 @@ class TestWatcher(TestCircus):
         watcher.reap_and_manage_processes()
 
         # we should have a new process here now
-        self.assertEquals(len(watcher.processes), 2)
+        self.assertEquals(len(watcher.processes), nb_proc)
         for p in watcher.processes.values():
             # and that one needs to have a new pid.
             self.assertFalse(p.pid in to_kill)
@@ -110,7 +128,8 @@ class TestWatcher(TestCircus):
 
         truncate_file(self.test_file)  # make sure we have a clean slate
         # expect at least one restart (max_age and restart), in less than 5s
-        self.assertTrue(poll_for(self.test_file, 'QUITSTART'))
+        self.assertTrue(poll_for(self.test_file,
+                                 ('QUITSTOPSTART', 'QUITSTART')))
         current_pids = self.pids()
         self.assertEqual(len(current_pids), 1)
         self.assertNotEqual(initial_pids, current_pids)
@@ -155,15 +174,34 @@ class TestWatcherInitialization(TestCircus):
         watcher.start()
         try:
             time.sleep(.1)
-            minor = sys.version_info[1]
-            wanted = os.path.join(venv, 'lib', 'python2.%d' % minor,
+            py_version = get_python_version()
+            major = py_version[0]
+            minor = py_version[1]
+            wanted = os.path.join(venv, 'lib', 'python%d.%d' % (major, minor),
                                   'site-packages',
-                                  'pip-7.7-py2.%d.egg' % minor)
+                                  'pip-7.7-py%d.%d.egg' % (major, minor))
             ppath = watcher.watcher.env['PYTHONPATH']
         finally:
             watcher.stop()
         self.assertTrue(wanted in ppath)
 
+    def test_venv_site_packages(self):
+        venv = os.path.join(os.path.dirname(__file__), 'venv')
+        watcher = SomeWatcher(virtualenv=venv)
+        watcher.start()
+        try:
+            time.sleep(.1)
+            py_version = get_python_version()
+            major = py_version[0]
+            minor = py_version[1]
+            wanted = os.path.join(venv, 'lib', 'python%d.%d' % (major, minor),
+                                  'site-packages')
+            ppath = watcher.watcher.env['PYTHONPATH']
+        finally:
+            watcher.stop()
+
+        self.assertTrue(wanted in ppath.split(os.pathsep))
+
 
 class SomeWatcher(threading.Thread):
 
@@ -211,9 +249,11 @@ class TestWatcherHooks(TestCircus):
 
     def run_with_hooks(self, hooks):
         self.stream = QueueStream()
+        self.errstream = QueueStream()
         dummy_process = 'circus.tests.support.run_process'
         return self._create_circus(dummy_process,
                                    stdout_stream={'stream': self.stream},
+                                   stderr_stream={'stream': self.errstream},
                                    hooks=hooks)
 
     def _stop(self):
@@ -241,6 +281,9 @@ class TestWatcherHooks(TestCircus):
 
             raise TypeError('beeeuuua')
 
+        old = logger.exception
+        logger.exception = lambda x: x
+
         hooks = {hook_name: (hook, False)}
         testfile, arbiter = self.run_with_hooks(hooks)
         try:
@@ -249,6 +292,7 @@ class TestWatcherHooks(TestCircus):
             self.assertEqual(self.get_status(), status)
         finally:
             arbiter.stop()
+            logger.exception = old
 
         self.assertTrue(events['before_start_called'])
         self.assertEqual(events['arbiter_in_hook'], arbiter)
@@ -267,8 +311,9 @@ class TestWatcherHooks(TestCircus):
         self._test_hooks(hook_name='after_start')
 
     def test_after_start_fails(self):
-        self._test_hooks(behavior=ERROR, status='stopped',
-                         hook_name='after_start')
+        with captured_output('stderr'):
+            self._test_hooks(behavior=ERROR, status='stopped',
+                             hook_name='after_start')
 
     def test_after_start_false(self):
         self._test_hooks(behavior=FAILURE, status='stopped',
@@ -279,9 +324,10 @@ class TestWatcherHooks(TestCircus):
                          call=self._stop)
 
     def test_before_stop_fails(self):
-        self._test_hooks(behavior=ERROR, status='stopped',
-                         hook_name='before_stop',
-                         call=self._stop)
+        with captured_output('stdout'):
+            self._test_hooks(behavior=ERROR, status='stopped',
+                             hook_name='before_stop',
+                             call=self._stop)
 
     def test_before_stop_false(self):
         self._test_hooks(behavior=FAILURE, status='stopped',
@@ -292,14 +338,28 @@ class TestWatcherHooks(TestCircus):
                          call=self._stop)
 
     def test_after_stop_fails(self):
-        self._test_hooks(behavior=ERROR, status='stopped',
-                         hook_name='after_stop',
-                         call=self._stop)
+        with captured_output('stdout'):
+            self._test_hooks(behavior=ERROR, status='stopped',
+                             hook_name='after_stop',
+                             call=self._stop)
 
     def test_after_stop_false(self):
         self._test_hooks(behavior=FAILURE, status='stopped',
                          hook_name='after_stop', call=self._stop)
 
+    def test_before_spawn(self):
+        self._test_hooks(hook_name='before_spawn')
+
+    def test_before_spawn_failure(self):
+        with captured_output('stdout'):
+            self._test_hooks(behavior=ERROR, status='stopped',
+                             hook_name='before_spawn',
+                             call=self._stop)
+
+    def test_before_spawn_false(self):
+        self._test_hooks(behavior=FAILURE, status='stopped',
+                         hook_name='before_spawn', call=self._stop)
+
 
 def oneshot_process(test_file):
     pass
