@@ -16,14 +16,12 @@ from ConfigParser import (ConfigParser, MissingSectionHeaderError,
 
 from psutil import AccessDenied, NoSuchProcess, Process
 
-from circus.py3compat import string_types
-
 
 # default endpoints
 DEFAULT_ENDPOINT_DEALER = "tcp://127.0.0.1:5555"
 DEFAULT_ENDPOINT_SUB = "tcp://127.0.0.1:5556"
 DEFAULT_ENDPOINT_STATS = "tcp://127.0.0.1:5557"
-DEFAULT_ENDPOINT_MULTICAST = "upd://237.219.251.97:12027"
+DEFAULT_ENDPOINT_MULTICAST = "udp://237.219.251.97:12027"
 
 
 try:
@@ -238,6 +236,8 @@ def to_uid(name):
         except KeyError:
             raise ValueError("%r isn't a valid user id" % name)
 
+    from circus.py3compat import string_types  # circular import fix
+
     if not isinstance(name, string_types):
         raise TypeError(name)
 
@@ -261,6 +261,8 @@ def to_gid(name):
         except (KeyError, OverflowError):
             raise ValueError("No such group: %r" % name)
 
+    from circus.py3compat import string_types  # circular import fix
+
     if not isinstance(name, string_types):
         raise TypeError(name)
 
@@ -302,6 +304,11 @@ def close_on_exec(fd):
     fcntl.fcntl(fd, fcntl.F_SETFD, flags)
 
 
+def get_python_version():
+    """Get a 3 element tuple with the python version"""
+    return sys.version_info[:3]
+
+
 INDENTATION_LEVEL = 0
 
 
@@ -644,21 +651,31 @@ def load_virtualenv(watcher):
             except OSError:
                 continue
 
+    py_path = watcher.env.get('PYTHONPATH')
+
     if venv_pkgs:
         venv_path = os.pathsep.join(venv_pkgs)
 
-        py_path = watcher.env.get('PYTHONPATH')
         if py_path:
             path = os.pathsep.join([venv_path, py_path])
         else:
             path = venv_path
 
-        watcher.env['PYTHONPATH'] = path
+    # Add watcher virtualenv site-packages dir to the python path
+    if path and not sitedir in path.split(os.pathsep):
+        path = os.pathsep.join([path, sitedir])
+    else:
+        if py_path:
+            path = os.pathsep.join([py_path, sitedir])
+        else:
+            path = sitedir
+
+    watcher.env['PYTHONPATH'] = path
 
 
 def create_udp_socket(mcast_addr, mcast_port):
     """Create an udp multicast socket for circusd cluster auto-discovery.
-    Mcas_addr must be between 224.0.0.0 and 239.255.255.255
+    mcast_addr must be between 224.0.0.0 and 239.255.255.255
     """
     try:
         ip_splitted = map(int, mcast_addr.split('.'))
