@@ -142,7 +142,7 @@ class Watcher(object):
       or the callabled itself and a boolean flag indicating if an
       exception occuring in the hook should not be ignored.
       Possible values for the hook name: *before_start*, *after_start*,
-      *before_stop*, *after_stop*.
+      *befor_spawn*, *before_stop*, *after_stop*.
 
     - **options** -- extra options for the worker. All options
       found in the configuration file for instance, are passed
@@ -200,7 +200,8 @@ class Watcher(object):
         self.max_age = int(max_age)
         self.max_age_variance = int(max_age_variance)
         self.ignore_hook_failure = ['before_stop', 'after_stop']
-        self.hooks = self._resolve_hooks(hooks)
+        self.hooks = {}
+        self._resolve_hooks(hooks)
         self.respawn = respawn
         self.autostart = autostart
         self.close_child_stdout = close_child_stdout
@@ -251,6 +252,29 @@ class Watcher(object):
         self.sockets = self.evpub_socket = None
         self.arbiter = None
 
+    def _reload_hook(self, key, hook, ignore_error):
+        hook_name = key.split('.')[-1]
+        self._resolve_hook(hook_name, hook, ignore_error)
+
+    def _reload_stream(self, key, val):
+        parts = key.split('.', 1)
+
+        if parts[0] == 'stdout':
+            self.stdout_stream_conf[parts[1]] = val
+            self.stdout_stream = get_stream(self.stdout_stream_conf)
+        else:
+            self.stderr_stream_conf[parts[1]] = val
+            self.stderr_stream = get_stream(self.stderr_stream_conf)
+
+        self._create_redirectors()
+        if self.stdout_redirector is not None:
+            self.stdout_redirector.start()
+
+        if self.stderr_redirector is not None:
+            self.stderr_redirector.start()
+
+        return 1
+
     def _create_redirectors(self):
         if self.stdout_stream:
             if self.stdout_redirector is not None:
@@ -268,35 +292,34 @@ class Watcher(object):
         else:
             self.stderr_redirector = None
 
+    def _resolve_hook(self, name, callable_or_name, ignore_failure):
+        if callable(callable_or_name):
+            self.hooks[name] = callable_or_name
+        else:
+            # will raise ImportError on failure
+            self.hooks[name] = resolve_name(callable_or_name)
+
+        if ignore_failure:
+            self.ignore_hook_failure.append(name)
+
     def _resolve_hooks(self, hooks):
         """Check the supplied hooks argument to make sure we can find
         callables"""
-        if not hooks:
-            return {}
-
-        resolved_hooks = {}
-
-        for hook_name, hook_value in hooks.items():
-            callable_or_name, ignore_failure = hook_value
-
-            if callable(callable_or_name):
-                resolved_hooks[hook_name] = callable_or_name
-            else:
-                # will raise ImportError on failure
-                resolved_hook = resolve_name(callable_or_name)
-                resolved_hooks[hook_name] = resolved_hook
-
-            if ignore_failure:
-                self.ignore_hook_failure.append(hook_name)
-
-        return resolved_hooks
+        if hooks is None:
+            return
+        for name, (callable_or_name, ignore_failure) in hooks.items():
+            self._resolve_hook(name, callable_or_name, ignore_failure)
 
     @classmethod
-    def load_from_config(cls, config, **extras):
+    def load_from_config(cls, config):
         if 'env' in config:
             config['env'] = parse_env_dict(config['env'])
-        config.update(extras)
-        return cls(name=config.pop('name'), cmd=config.pop('cmd'), **config)
+        cfg = config.copy()
+
+        w = cls(name=config.pop('name'), cmd=config.pop('cmd'), **config)
+        w._cfg = cfg
+
+        return w
 
     @util.debuglog
     def initialize(self, evpub_socket, sockets, arbiter):
@@ -340,6 +363,12 @@ class Watcher(object):
                     elif e.errno == errno.ECHILD:
                         # nothing to do here, we do not have any child
                         # process running
+                        # but we still need to send the "reap" signal.
+                        logger.debug('reaping already dead process %s [%s]',
+                                     pid, self.name)
+                        self.notify_event(
+                            "reap",
+                            {"process_pid": pid, "time": time.time()})
                         return
                     else:
                         raise
@@ -359,7 +388,7 @@ class Watcher(object):
         if process.status in (DEAD_OR_ZOMBIE, UNEXISTING):
             process.stop()
 
-        logger.debug('reaping process %s [%s]' % (pid, self.name))
+        logger.debug('reaping process %s [%s]', pid, self.name)
         self.notify_event("reap", {"process_pid": pid, "time": time.time()})
 
     @util.debuglog
@@ -439,7 +468,12 @@ class Watcher(object):
         if self.stopped:
             return
 
-        cmd = util.replace_gnu_args(self.cmd, sockets=self._get_sockets_fds())
+        if not self.call_hook('before_spawn'):
+            self.stopped = True
+            return False
+
+        cmd = util.replace_gnu_args(self.cmd, sockets=self._get_sockets_fds(),
+                                    env=self.env)
         self._process_counter += 1
         nb_tries = 0
         pipe_stdout = self.stdout_redirector is not None
@@ -473,7 +507,7 @@ class Watcher(object):
                 self.processes[process.pid] = process
                 logger.debug('running %s process [pid %d]', self.name,
                              process.pid)
-            except OSError, e:
+            except OSError as e:
                 logger.warning('error in %r: %s', self.name, str(e))
 
             if process is None:
@@ -491,10 +525,10 @@ class Watcher(object):
         """Kill process.
         """
         # remove redirections
-        if self.stdout_redirector is not None:
+        if self.stdout_redirector is not None and process.stdout is not None:
             self.stdout_redirector.remove_redirection(process.stdout)
 
-        if self.stderr_redirector is not None:
+        if self.stderr_redirector is not None and process.stderr is not None:
             self.stderr_redirector.remove_redirection(process.stderr)
 
         logger.debug("%s: kill process %s", self.name, process.pid)
@@ -715,21 +749,24 @@ class Watcher(object):
         self.notify_event("reload", {"time": time.time()})
         logger.info('%s reloaded', self.name)
 
-    @util.debuglog
-    def incr(self, nb=1):
-        if self.singleton and self.numprocesses == 1:
+    def set_numprocesses(self, np):
+        if np < 0:
+            np = 0
+        if self.singleton and np > 1:
             raise ValueError('Singleton watcher has a single process')
 
-        self.numprocesses += nb
+        self.numprocesses = np
         self.manage_processes()
+
         return self.numprocesses
 
+    @util.debuglog
+    def incr(self, nb=1):
+        return self.set_numprocesses(self.numprocesses + nb)
+
     @util.debuglog
     def decr(self, nb=1):
-        if self.numprocesses > 0:
-            self.numprocesses -= nb
-            self.manage_processes()
-        return self.numprocesses
+        return self.set_numprocesses(self.numprocesses - nb)
 
     def set_opt(self, key, val):
         """Set a watcher option.
@@ -740,7 +777,6 @@ class Watcher(object):
         - 0: trigger the process management
         - 1: trigger a graceful reload of the processes;
         """
-
         action = 0
 
         if key in self._options:
@@ -748,6 +784,8 @@ class Watcher(object):
             action = -1    # XXX for now does not trigger a reload
         elif key == "numprocesses":
             val = int(val)
+            if val < 0:
+                val = 0
             if self.singleton and val > 1:
                 raise ValueError('Singleton watcher has a single process')
             self.numprocesses = val
@@ -782,6 +820,18 @@ class Watcher(object):
         elif key == "max_age_variance":
             self.max_age_variance = int(val)
             action = 1
+        elif (key.startswith('stdout_stream') or
+              key.startswith('stderr_stream')):
+            action = self._reload_stream(key, val)
+        elif key.startswith('hook'):
+            val = val.split(',')
+            if len(val) == 2:
+                ignore_error = util.to_bool(val[1])
+            else:
+                ignore_error = False
+            hook = val[0]
+            self._reload_hook(key, hook, ignore_error)
+            action = 1
 
         # send update event
         self.notify_event("updated", {"time": time.time()})
